"""PostProcessor for customizing code language css class."""

# ----------------------------------------------------------------------------
# Copyright (c) the IPython Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Imports
# ----------------------------------------------------------------------------
from __future__ import print_function

import io
import re

from nbconvert.postprocessors.base import PostProcessorBase
from traitlets import Unicode

try:
    from html.parser import HTMLParser  # py3
except:
    from HTMLParser import HTMLParser  # py2


class HtmlHighlightStripper(HTMLParser):
    """
    An object that is fed with html and removes <span> tags
    enclosed inside <pre> or <code> tags. This effectively removes
    static highlighting, but can strip other things too. This class is
    made to work with pandoc/marked/pygments type of output
    """

    def __init__(self, ostream):
        HTMLParser.__init__(self)
        self.in_pre = False
        self.in_code = False
        self.ostream = ostream
        self.pygments_fix = False

    def handle_starttag(self, tag, attrs):
        _print = True
        if tag == u"pre":
            if self.pygments_fix:
                _print = False
            self.in_pre = True
        elif tag == u"code":
            self.in_code = True
        elif tag == u"div":
            # Pygments doesn't protect raw code inside a <code> el.
            # and the language is in the enclosing <div> so we must
            # do some transfos.
            attrs_dict = dict(attrs)
            cssclass = attrs_dict.pop(u"class", None)
            if cssclass and cssclass.startswith(u"hl-"):
                self.pygments_fix = True
                tag = u"div" + self.stringify_attrs(attrs_dict) + ">"
                tag += u"<pre><code"

        elif self.in_pre or self.in_code:
            _print = False

        if _print:
            self.out(u"<%s%s>" % (tag, self.stringify_attrs(attrs)))

    def handle_endtag(self, tag):
        _print = True
        if tag == u"pre":
            if self.pygments_fix:
                _print = False
            self.in_pre = False
        elif tag == u"code":
            self.in_code = False
        elif self.in_pre or self.in_code:
            _print = False
        elif tag == u"div" and self.pygments_fix:
            tag = u"code></pre></div"
            self.pygments_fix = False

        if _print:
            self.out(u"</" + tag + ">")

    def def_handle(self, arg):
        self.out(arg)

    def handle_entityref(self, arg):
        self.out("&" + arg + ";")

    def handle_decl(self, arg):
        self.out("<!" + arg + ">")

    def handle_charref(self, arg):
        self.out("&#" + arg + ";")

    def handle_comment(self, arg):
        self.out("<!--" + arg + "-->")

    def handle_pi(self, arg):
        self.out("<?" + arg + ">")

    handle_data = unknown_decl = def_handle

    def stringify_attrs(self, attrs):
        if not attrs:
            return ""
        return " " + " ".join([k + "=" + '"%s"' % v for k, v in attrs])

    def out(self, data):
        self.ostream.write(data)


def rec(reg):
    return re.compile(reg, re.S)


pandoc_code_markup_re = rec(r'<pre\s*class="(\w*?)">\s*<code>')
marked_code_markup_re = rec(r'<pre>\s*<code\s*class="language-(\w*)">')
pygments_code_markup_re = rec(r'<pre>\s*<code\s*class="hl-(\w*)">')


class JsHighlightPostProcessor(PostProcessorBase):
    """
    Customize CSS classes of code blocks to let
    your favourite JS highlighting engine pick them up.
    """

    css_substitution = Unicode(default_value='{lang}', config=True,
                               help="""A template string to insert into the <pre>
        tags' ``class`` attribute. The only substituted value
        is {lang} where ``lang`` is the language of the fenced
        code block.""")

    def postprocess(self, input):

        _html = None
        with io.open(input, "r", encoding="utf-8") as f:
            _html = f.read()

        stripped = io.StringIO()
        stripper = HtmlHighlightStripper(stripped)
        stripper.feed(_html)
        _html = stripped.getvalue()

        # Prepare substitution string. {lang} is a placeholder
        # for the language detected by the regexp, which is the first
        # captured element.
        css = self.css_substitution.format(lang=r"\1")
        substitute = '<pre class="' + css + '"><code>'

        for cre in (pandoc_code_markup_re,
                    marked_code_markup_re,
                    pygments_code_markup_re):
            _html = cre.sub(substitute, _html)

        with io.open(input, "w", encoding="utf-8") as f:
            f.write(_html)

        return


msg = """
Modify code blocks in nbconvert's HTML output so that the highlighting
is done in the browser by Javascript.

Usage:
======
python js_highlight.py file.html css_substitution

Example (to let google-prettify do the highlighting):
=====================================================
    nbconvert MyNotebook.ipynb
    python js_highlight.py MyNotebook.html "prettyprint lang_{lang}"

Details
=======
"""


def usage():
    print(msg)
    print(JsHighlightPostProcessor.class_get_help())


def main(path, substitution=None):
    """allow running the module to customize css classes of html code blocks"""
    htmlClassCustomizer = JsHighlightPostProcessor()
    if substitution is not None:
        htmlClassCustomizer.css_substitution = substitution
    htmlClassCustomizer(path)


if __name__ == '__main__':
    import sys
    if len(sys.argv) < 3:
        usage()
        sys.exit(-1)
    main(*sys.argv[1:])



# -*- coding: utf-8 -*-
"""API to install/remove all jupyter_contrib_nbextensions."""

from __future__ import (
    absolute_import, division, print_function, unicode_literals,
)

import errno
import os

import jupyter_highlight_selected_word
import latex_envs
import psutil
from jupyter_contrib_core.notebook_compat import nbextensions
from jupyter_nbextensions_configurator.application import \
    EnableJupyterNbextensionsConfiguratorApp
from traitlets.config import Config
from traitlets.config.manager import BaseJSONConfigManager

import jupyter_contrib_nbextensions.nbconvert_support


class NotebookRunningError(Exception):
    pass


def notebook_is_running():
    """Return true if a notebook process appears to be running."""
    for p in psutil.process_iter():
        # p.name() can throw exceptions due to zombie processes on Mac OS X, so
        # ignore psutil.ZombieProcess
        # (See https://code.google.com/p/psutil/issues/detail?id=428)

        # It isn't enough to search just the process name, we have to
        # search the process command to see if jupyter-notebook is running.

        # Checking the process command can cause an AccessDenied exception to
        # be thrown for system owned processes, ignore those as well
        try:
            # use lower, since python may be Python, e.g. on OSX
            if ('python' or 'jupyter') in p.name().lower():
                for arg in p.cmdline():
                    # the missing k is deliberate!
                    # The usual string 'jupyter-notebook' can get truncated.
                    if 'jupyter-noteboo' in arg:
                        return True
        except (psutil.ZombieProcess, psutil.AccessDenied):
            pass
        return False


def toggle_install(install, user=False, sys_prefix=False, overwrite=False,
                   symlink=False, prefix=None, nbextensions_dir=None,
                   logger=None):
    """Install or remove all jupyter_contrib_nbextensions files & config."""
    if notebook_is_running():
        raise NotebookRunningError(
            'Cannot configure while the Jupyter notebook server is running')
    _check_conflicting_kwargs(user=user, sys_prefix=sys_prefix, prefix=prefix,
                              nbextensions_dir=nbextensions_dir)
    toggle_install_files(
        install, user=user, sys_prefix=sys_prefix, overwrite=overwrite,
        symlink=symlink, prefix=prefix, nbextensions_dir=nbextensions_dir,
        logger=logger)
    toggle_install_config(
        install, user=user, sys_prefix=sys_prefix, logger=logger)


def toggle_install_files(install, user=False, sys_prefix=False, logger=None,
                         overwrite=False, symlink=False, prefix=None,
                         nbextensions_dir=None):
    """Install/remove jupyter_contrib_nbextensions files."""
    if notebook_is_running():
        raise NotebookRunningError(
            'Cannot configure while the Jupyter notebook server is running')
    kwargs = dict(user=user, sys_prefix=sys_prefix, prefix=prefix,
                  nbextensions_dir=nbextensions_dir)
    _check_conflicting_kwargs(**kwargs)
    kwargs['logger'] = logger
    if logger:
        logger.info(
            '{} jupyter_contrib_nbextensions nbextension files {} {}'.format(
                'Installing' if install else 'Uninstalling',
                'to' if install else 'from',
                'jupyter data directory'))
    component_nbext_packages = [
        jupyter_contrib_nbextensions,
        jupyter_highlight_selected_word,
        latex_envs,
    ]
    for mod in component_nbext_packages:
        if install:
            nbextensions.install_nbextension_python(
                mod.__name__, overwrite=overwrite, symlink=symlink, **kwargs)
        else:
            nbextensions.uninstall_nbextension_python(mod.__name__, **kwargs)


def toggle_install_config(install, user=False, sys_prefix=False, logger=None):
    """Install/remove contrib nbextensions to/from jupyter_nbconvert_config."""
    if notebook_is_running():
        raise NotebookRunningError(
            'Cannot configure while the Jupyter notebook server is running')
    _check_conflicting_kwargs(user=user, sys_prefix=sys_prefix)
    config_dir = nbextensions._get_config_dir(user=user, sys_prefix=sys_prefix)
    if logger:
        logger.info(
            '{} jupyter_contrib_nbextensions items {} config in {}'.format(
                'Installing' if install else 'Uninstalling',
                'to' if install else 'from',
                config_dir))

    # Configure the jupyter_nbextensions_configurator serverextension to load
    if install:
        configurator_app = EnableJupyterNbextensionsConfiguratorApp(
            user=user, sys_prefix=sys_prefix, logger=logger)
        configurator_app.start()
        nbextensions.enable_nbextension(
            'notebook', 'contrib_nbextensions_help_item/main',
            user=user, sys_prefix=sys_prefix, logger=logger)
    else:
        nbconf_cm = BaseJSONConfigManager(
            config_dir=os.path.join(config_dir, 'nbconfig'))
        for require, section in {
                'contrib_nbextensions_help_item/main': 'notebook'}.items():
            if logger:
                logger.info('- Disabling {}'.format(require))
                logger.info(
                    '--  Editing config: {}'.format(
                        nbconf_cm.file_name(section)))
        # disabled_conf['load_extensions'][require] = None
        nbconf_cm.update('notebook', {'load_extensions': {require: None}})

    # Set extra template path, pre- and post-processors for nbconvert
    cm = BaseJSONConfigManager(config_dir=config_dir)
    config_basename = 'jupyter_nbconvert_config'
    config = cm.get(config_basename)
    # avoid warnings about unset version
    config.setdefault('version', 1)
    if logger:
        logger.info(
            u'- Editing config: {}'.format(cm.file_name(config_basename)))

    # Set extra template path, pre- and post-processors for nbconvert
    if logger:
        logger.info('--  Configuring nbconvert template path')
    # our templates directory
    _update_config_list(config, 'Exporter.template_path', [
        '.',
        jupyter_contrib_nbextensions.nbconvert_support.templates_directory(),
    ], install)
    # our preprocessors
    if logger:
        logger.info('--  Configuring nbconvert preprocessors')
    proc_mod = 'jupyter_contrib_nbextensions.nbconvert_support'
    _update_config_list(config, 'Exporter.preprocessors', [
        proc_mod + '.CodeFoldingPreprocessor',
        proc_mod + '.PyMarkdownPreprocessor',
    ], install)
    if logger:
        logger.info(
            u'- Writing config: {}'.format(cm.file_name(config_basename)))
    _set_managed_config(cm, config_basename, config, logger=logger)


def install(user=False, sys_prefix=False, prefix=None, nbextensions_dir=None,
            logger=None, overwrite=False, symlink=False):
    """Install all jupyter_contrib_nbextensions files & config."""
    return toggle_install(
        True, user=user, sys_prefix=sys_prefix, prefix=prefix,
        nbextensions_dir=nbextensions_dir, logger=logger,
        overwrite=overwrite, symlink=symlink)


def uninstall(user=False, sys_prefix=False, prefix=None, nbextensions_dir=None,
              logger=None):
    """Uninstall all jupyter_contrib_nbextensions files & config."""
    return toggle_install(
        False, user=user, sys_prefix=sys_prefix, prefix=prefix,
        nbextensions_dir=nbextensions_dir, logger=logger)

# -----------------------------------------------------------------------------
# Private API
# -----------------------------------------------------------------------------


def _check_conflicting_kwargs(**kwargs):
    if sum(map(bool, kwargs.values())) > 1:
        raise nbextensions.ArgumentConflict(
            "Cannot specify more than one of {}.\nBut recieved {}".format(
                ', '.join(kwargs.keys),
                ', '.join(['{}={}'.format(k, v)
                           for k, v in kwargs.items() if v])))


def _set_managed_config(cm, config_basename, config, logger=None):
    """Write config owned by the given config manager, removing if empty."""
    config_path = cm.file_name(config_basename)
    msg = 'config file {}'.format(config_path)
    if len(config) > ('version' in config):
        if logger:
            logger.info('--  Writing updated {}'.format(msg))
        # use set to ensure removed keys get removed
        cm.set(config_basename, config)
    else:
        if logger:
            logger.info('--  Removing now-empty {}'.format(msg))
        try:
            os.remove(config_path)
        except OSError as ex:
            if ex.errno != errno.ENOENT:
                raise


def _update_config_list(config, list_key, values, insert):
    """
    Add or remove items as required to/from a config value which is a list.

    This exists in order to avoid clobbering values other than those which we
    wish to add/remove, and to neatly remove a list when it ends up empty.
    """
    section, list_key = list_key.split('.')
    conf_list = config.setdefault(section, Config()).setdefault(list_key, [])
    list_alteration_method = 'append' if insert else 'remove'
    for val in values:
        if (val in conf_list) != insert:
            getattr(conf_list, list_alteration_method)(val)
    if not insert:
        # remove empty list
        if len(conf_list) == 0:
            config[section].pop(list_key)
        # remove empty section
        if len(config[section]) == 0:
            config.pop(section)



"""Embed graphics into HTML Exporter class"""

import base64
import re
from nbconvert.exporters.html import HTMLExporter

try:
    from urllib.request import urlopen  # py3
except ImportError:
    from urllib2 import urlopen


class EmbedHTMLExporter(HTMLExporter):
    """
    :mod:`nbconvert` Exporter which embeds graphics as base64 into html.

    Convert to HTML and embed graphics (pdf, svg and raster images) in the HTML
    file.

    Example usage::

        jupyter nbconvert --to html_embed mynotebook.ipynb
    """

    def replfunc(self, match):
        """Replace source url or file link with base64 encoded blob."""
        url = match.group(1)
        imgformat = url.split('.')[-1]
        if url.startswith('http'):
            data = urlopen(url).read()
        elif url.startswith('data'):
            img = '<img src="' + url + '"'
            return img
        else:
            with open(url, 'rb') as f:
                data = f.read()

        self.log.info("embedding url: %s, format: %s" % (url, imgformat))
        b64_data = base64.b64encode(data).decode("utf-8")
        if imgformat == "svg":
            img = '<img src="data:image/svg+xml;base64,' + \
                b64_data + '"'
        elif imgformat == "pdf":
            img = '<img src="data:application/pdf;base64,' + \
                b64_data + '"'
        else:
            img = '<img src="data:image/' + imgformat + \
                ';base64,' + b64_data + '"'
        return img

    def from_notebook_node(self, nb, resources=None, **kw):
        output, resources = super(
            EmbedHTMLExporter, self).from_notebook_node(nb, resources)

        regex = re.compile('<img\s+src="([^"]+)"')

        embedded_output = regex.sub(self.replfunc, output)
        return embedded_output, resources



"""Nbconvert exporter to inline css & js for collapsible_headings."""

import json
import os

from notebook.services.config import ConfigManager

from jupyter_contrib_nbextensions import __file__ as contrib_init
from .exporter_inliner import ExporterInliner


class ExporterCollapsibleHeadings(ExporterInliner):
    """
    HTMLExporter which inlines the collapsible_headings nbextension.

    Export collapsible_headings nbextension functionality to html
    by inlining relevant css and js content.

    Example usage::

        jupyter nbconvert --to html_ch FILE.ipynb
    """

    def _template_file_default(self):
        return 'collapsible_headings'

    def __init__(self, *args, **kwargs):
        super(ExporterCollapsibleHeadings, self).__init__(*args, **kwargs)

        ch_dir = os.path.join(
            os.path.dirname(contrib_init), 'nbextensions',
            'collapsible_headings')

        with open(os.path.join(ch_dir, 'main.css'), 'r') as f:
            main_css = f.read()
        self.inliner_resources['css'].append(main_css)

        self.inliner_resources['css'].append("""
/* no local copies of fontawesome fonts from basic templates, so get them from cdn */
@font-face {
    font-family: 'FontAwesome';
    src: url('https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.2.0/fonts/fontawesome-webfont.eot');
    src: url('https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.2.0/fonts/fontawesome-webfont.eot?#iefix') format('embedded-opentype'),
         url('https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.2.0/fonts/fontawesome-webfont.woff') format('woff'),
         url('https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.2.0/fonts/fontawesome-webfont.ttf') format('truetype'),
         url('https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.2.0/fonts/fontawesome-webfont.svg#fontawesomeregular') format('svg');
    font-weight: normal;
    font-style: normal;
}
""")  # noqa: E501

        with open(os.path.join(ch_dir, 'main.js'), 'r') as f:
            self.inliner_resources['js'].append(
                f.read().replace(
                    "define([",
                    "define('nbextensions/collapsible_headings/main', [")
            )

        cm = ConfigManager()
        collapsible_headings_options = cm.get('notebook').get(
            'collapsible_headings', {})
        self.inliner_resources['js'].append("""
require([
    'jquery',
    'nbextensions/collapsible_headings/main'
], function (
    $,
    nbext
) {
    nbext.set_collapsible_headings_options(%s);
    $(document).ready(function () {
        nbext.refresh_all_headings();
    });
});
""" % json.dumps(collapsible_headings_options))



from __future__ import print_function

import json
from sys import getsizeof

from IPython import get_ipython
from IPython.core.magics.namespace import NamespaceMagics

_nms = NamespaceMagics()
_Jupyter = get_ipython()
_nms.shell = _Jupyter.kernel.shell


def _getsizeof(x):
    # return the size of variable x. Amended version of sys.getsizeof
    # which also supports ndarray, Series and DataFrame
    if type(x).__name__ in ['ndarray', 'Series']:
        return x.nbytes
    elif type(x).__name__ == 'DataFrame':
        return x.memory_usage().sum()
    else:
        return getsizeof(x)


def var_dic_list():
    types_to_exclude = ['module', 'function', 'builtin_function_or_method',
                        'instance', '_Feature']
    values = _nms.who_ls()
    vardic = [{'varName': v, 'varType': type(eval(v)).__name__, 'varSize': _getsizeof(eval(v)), 'varContent': str(eval(v))[:200]} # noqa
    for v in values if (v not in ['_html', '_nms', 'NamespaceMagics', '_Jupyter']) & (type(eval(v)).__name__ not in types_to_exclude)] # noqa 
    return json.dumps(vardic)


# command to refresh the list of variables
print(var_dic_list())
